//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "psocapi.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00		// LED mode
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"RGB Color Sensor",	// user device name
	"Frame Number",		// user name Channel 0
	"R",				// user name Channel 1
	"G",				// user name Channel 2
	"B"					// user name Channel 3
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// function Prototypes
//----------------------------------------
static void Goto_I2C(void);
static void Goto_SPI(void);

//----------------------------------------
// constant definitions
//----------------------------------------
// ADJD-S313 device addresses
#define ADDR_S313	0x58

// ADJD-S313 register addresses
#define ADDR_ACQ	0x02

#define ADDR_SETUP0	0x03
#define ADDR_SETUP1	0x04
#define ADDR_SETUP2	0x0C
#define ADDR_SETUP3	0x0D
#define ADDR_SETUP4	0x0E

#define ADDR_PDASB	0x09
#define ADDR_PDASG	0x0A
#define ADDR_PDASR	0x0B

#define ADDR_TINTB	0x0F
#define ADDR_TINTG	0x10
#define ADDR_TINTR	0x11

#define ADDR_ADCB	0x42
#define ADDR_ADCG	0x43
#define ADDR_ADCR	0x44

//----------------------------------------
// constant lookup-table for ADC->physical value conversion
//----------------------------------------

//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eReset,
	eIdle,
	eReadADC
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

static BYTE		PDASR, PDASG, PDASB;	// TODO: sensor optimization to define these six variables
static BYTE		TINTR, TINTG, TINTB;

BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM1_2, DM1_1, DM1_0;
BYTE			DM2_2, DM2_1, DM2_0;

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	FSM = eReset;

	// turn on power for the sensor
	GPIO_LOW(SLEEP)
	// deassert Reset
	GPIO_HIGH(XRST)

	// initialize the Cypress' I2C interface
	I2CHW_Start();
	I2CHW_EnableMstr();
}
//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	// turn off power for the sensor
	GPIO_HIGH(SLEEP)

	// disable I2C hardware
	I2CHW_DisableMstr();

	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM1_2 = PRT1DM2;
	DM1_1 = PRT1DM1;
	DM1_0 = PRT1DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// OFF_nON is on Port0, all other signals on Port2
/*	PRT0DM2 |= ~OFF_nON_MASK;
	PRT0DM1 |= ~OFF_nON_MASK;
	PRT0DM0 &=  OFF_nON_MASK;
	PRT1DM2  = 0xFF;
	PRT1DM1  = 0xFF;
	PRT1DM0  = 0x00;
	PRT2DM2  = 0xFF;
	PRT2DM1  = 0xFF;
	PRT2DM0  = 0x00;
*/
}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	BYTE ReadMode;
	BYTE i;
		
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;
	PRT1DM2 = DM1_2;
	PRT1DM1 = DM1_1;
	PRT1DM0 = DM1_0;
	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// turn on power for the sensor
	GPIO_LOW(SLEEP)
	// Deassert reset
	GPIO_HIGH(XRST)
	I2CHW_EnableMstr();

	FSM = eReset;
}


//-----------------------------------------------------------------------------
// Read 8-bit register on I2C
//-----------------------------------------------------------------------------
BYTE I2C_ReadReg(BYTE Address) 
{
	BYTE fAck;
	BYTE Data;

	fAck = I2CHW_fSendStart      (ADDR_S313, I2CHW_WRITE);
	fAck = I2CHW_fWrite          (Address);
	fAck = I2CHW_fSendRepeatStart(ADDR_S313, I2CHW_READ);
	Data = I2CHW_bRead           (I2CHW_NAKslave);
	       I2CHW_SendStop();	
	
	return Data;
}


//-----------------------------------------------------------------------------
// Write 8-bit register on I2C
//-----------------------------------------------------------------------------
void I2C_WriteReg(BYTE Address, BYTE Data) 
{
	BYTE fAck;

	fAck = I2CHW_fSendStart      (ADDR_S313, I2CHW_WRITE);
	fAck = I2CHW_fWrite          (Address);
	fAck = I2CHW_fWrite          (Data);
	       I2CHW_SendStop();	
}


//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	#define NofReg 15
	static const BYTE ValidAddr[] = {0x02, 
							  0x03, 0x04, 0x0C, 0x0D, 0x0E, 
							  0x09, 0x0A, 0x0B, 
							  0x0F, 0x10, 0x11, 
							  0x42, 0x43, 0x44};	
	BYTE i;
	
	if ( (FEATURE_RPT.OUT.Target == eSensor) && (FEATURE_RPT.OUT.ParamSize == 1) )
	{
		for (i=0; i<NofReg; i++)
			// check if requested address is a valid S313 register
			if (FEATURE_RPT.OUT.Index_LSB == ValidAddr[i])
			{
				if (FEATURE_RPT.OUT.Get_nSet)
				{ // GET
					FEATURE_RPT.IN.Param[0] = I2C_ReadReg(FEATURE_RPT.OUT.Index_LSB);
					return 1;
				}
				else
				{ // SET
				 	I2C_WriteReg(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Param[0]);
				 	return 0;
				}
			}
	}
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	*NumberOfReturnedBytes = 0;
}

//-----------------------------------------------------------------------------
// Perform a reset of the ADJD-S313 sensor
//-----------------------------------------------------------------------------
void Sensor_Reset()
{
	static const BYTE SetupAddr[5] = {ADDR_SETUP0, ADDR_SETUP1, ADDR_SETUP2, ADDR_SETUP3, ADDR_SETUP4};
	BYTE i;
	
	
	// -- Reset pulse (active low)
	GPIO_LOW(XRST)
	for (i=0; i<10; i++);	// delay at least 10us
	GPIO_HIGH(XRST)
	for (i=0; i<10; i++);	// delay at least 10us
	
	// -- SETUP registers
	for (i=0; i<5; i++)
		I2C_WriteReg(SetupAddr[i], 0x01);
}

//-----------------------------------------------------------------------------
// Set Photodiode size and integration time
//-----------------------------------------------------------------------------
void Sensor_SetGain(BYTE PDASR, BYTE PDASG, BYTE PDASB,  BYTE TINTR, BYTE TINTG, BYTE TINTB)
{
	// -- PDAS registers
	I2C_WriteReg(ADDR_PDASR, PDASR);
	I2C_WriteReg(ADDR_PDASG, PDASG);
	I2C_WriteReg(ADDR_PDASB, PDASB);

	// -- TINT registers
	I2C_WriteReg(ADDR_TINTR, TINTR);
	I2C_WriteReg(ADDR_TINTR, TINTG);
	I2C_WriteReg(ADDR_TINTR, TINTB);
}

//-----------------------------------------------------------------------------
// Read R, G, B ADC registers
//-----------------------------------------------------------------------------
void Sensor_ReadRGB(BYTE *R, BYTE *G, BYTE *B)
{
	// -- Start A2D conversion, wait for the end of this
	I2C_WriteReg(ADDR_ACQ, 0x02);
	while (I2C_ReadReg(ADDR_ACQ) != 0x00);
	
	// -- Read ADC values
	*R = I2C_ReadReg(ADDR_ADCR);
	*G = I2C_ReadReg(ADDR_ADCG);
	*B = I2C_ReadReg(ADDR_ADCB);
}

//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the Sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() 
{
	static BYTE R,G,B;
	BYTE i;

	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case eReset:
			// activate Reset (active low)
			GPIO_LOW(XRST)
			FSM = eIdle;
			Sensor_Reset();
//			Sensor_SetGain(PDASR, PDASG, PDASB,  TINTR, TINTG, TINTB);
			break;
			
		case eIdle: 
			// deassert Reset
			GPIO_HIGH(XRST)
			if (Countdown_1ms == 0)
			{
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);;
				FSM = eReadADC;					  
			}
			break;			
			
		case eReadADC:
			Sensor_ReadRGB(&R, &G, &B);

			Report.Ch1_LSB = R;
			Report.Ch1_MSB = 0x00;

			Report.Ch2_LSB = G;
			Report.Ch2_MSB = 0x00;

			Report.Ch3_LSB = B;
			Report.Ch3_MSB = 0x00;

			NewSample = TRUE;

			FSM = eIdle;
			break;			
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
	/* currently no debug function */
}